// Code generated by ent, DO NOT EDIT.

package ent

import (
	"context"
	"errors"
	"fmt"
	"log"
	"reflect"

	"leafdev.top/leaf/rag/ent/migrate"

	"entgo.io/ent"
	"entgo.io/ent/dialect"
	"entgo.io/ent/dialect/sql"
	"entgo.io/ent/dialect/sql/sqlgraph"
	"leafdev.top/leaf/rag/ent/document"
	"leafdev.top/leaf/rag/ent/documentblock"
	"leafdev.top/leaf/rag/ent/library"
)

// Client is the client that holds all ent builders.
type Client struct {
	config
	// Schema is the client for creating, migrating and dropping schema.
	Schema *migrate.Schema
	// Document is the client for interacting with the Document builders.
	Document *DocumentClient
	// DocumentBlock is the client for interacting with the DocumentBlock builders.
	DocumentBlock *DocumentBlockClient
	// Library is the client for interacting with the Library builders.
	Library *LibraryClient
}

// NewClient creates a new client configured with the given options.
func NewClient(opts ...Option) *Client {
	client := &Client{config: newConfig(opts...)}
	client.init()
	return client
}

func (c *Client) init() {
	c.Schema = migrate.NewSchema(c.driver)
	c.Document = NewDocumentClient(c.config)
	c.DocumentBlock = NewDocumentBlockClient(c.config)
	c.Library = NewLibraryClient(c.config)
}

type (
	// config is the configuration for the client and its builder.
	config struct {
		// driver used for executing database requests.
		driver dialect.Driver
		// debug enable a debug logging.
		debug bool
		// log used for logging on debug mode.
		log func(...any)
		// hooks to execute on mutations.
		hooks *hooks
		// interceptors to execute on queries.
		inters *inters
	}
	// Option function to configure the client.
	Option func(*config)
)

// newConfig creates a new config for the client.
func newConfig(opts ...Option) config {
	cfg := config{log: log.Println, hooks: &hooks{}, inters: &inters{}}
	cfg.options(opts...)
	return cfg
}

// options applies the options on the config object.
func (c *config) options(opts ...Option) {
	for _, opt := range opts {
		opt(c)
	}
	if c.debug {
		c.driver = dialect.Debug(c.driver, c.log)
	}
}

// Debug enables debug logging on the ent.Driver.
func Debug() Option {
	return func(c *config) {
		c.debug = true
	}
}

// Log sets the logging function for debug mode.
func Log(fn func(...any)) Option {
	return func(c *config) {
		c.log = fn
	}
}

// Driver configures the client driver.
func Driver(driver dialect.Driver) Option {
	return func(c *config) {
		c.driver = driver
	}
}

// Open opens a database/sql.DB specified by the driver name and
// the data source name, and returns a new client attached to it.
// Optional parameters can be added for configuring the client.
func Open(driverName, dataSourceName string, options ...Option) (*Client, error) {
	switch driverName {
	case dialect.MySQL, dialect.Postgres, dialect.SQLite:
		drv, err := sql.Open(driverName, dataSourceName)
		if err != nil {
			return nil, err
		}
		return NewClient(append(options, Driver(drv))...), nil
	default:
		return nil, fmt.Errorf("unsupported driver: %q", driverName)
	}
}

// ErrTxStarted is returned when trying to start a new transaction from a transactional client.
var ErrTxStarted = errors.New("ent: cannot start a transaction within a transaction")

// Tx returns a new transactional client. The provided context
// is used until the transaction is committed or rolled back.
func (c *Client) Tx(ctx context.Context) (*Tx, error) {
	if _, ok := c.driver.(*txDriver); ok {
		return nil, ErrTxStarted
	}
	tx, err := newTx(ctx, c.driver)
	if err != nil {
		return nil, fmt.Errorf("ent: starting a transaction: %w", err)
	}
	cfg := c.config
	cfg.driver = tx
	return &Tx{
		ctx:           ctx,
		config:        cfg,
		Document:      NewDocumentClient(cfg),
		DocumentBlock: NewDocumentBlockClient(cfg),
		Library:       NewLibraryClient(cfg),
	}, nil
}

// BeginTx returns a transactional client with specified options.
func (c *Client) BeginTx(ctx context.Context, opts *sql.TxOptions) (*Tx, error) {
	if _, ok := c.driver.(*txDriver); ok {
		return nil, errors.New("ent: cannot start a transaction within a transaction")
	}
	tx, err := c.driver.(interface {
		BeginTx(context.Context, *sql.TxOptions) (dialect.Tx, error)
	}).BeginTx(ctx, opts)
	if err != nil {
		return nil, fmt.Errorf("ent: starting a transaction: %w", err)
	}
	cfg := c.config
	cfg.driver = &txDriver{tx: tx, drv: c.driver}
	return &Tx{
		ctx:           ctx,
		config:        cfg,
		Document:      NewDocumentClient(cfg),
		DocumentBlock: NewDocumentBlockClient(cfg),
		Library:       NewLibraryClient(cfg),
	}, nil
}

// Debug returns a new debug-client. It's used to get verbose logging on specific operations.
//
//	client.Debug().
//		Document.
//		Query().
//		Count(ctx)
func (c *Client) Debug() *Client {
	if c.debug {
		return c
	}
	cfg := c.config
	cfg.driver = dialect.Debug(c.driver, c.log)
	client := &Client{config: cfg}
	client.init()
	return client
}

// Close closes the database connection and prevents new queries from starting.
func (c *Client) Close() error {
	return c.driver.Close()
}

// Use adds the mutation hooks to all the entity clients.
// In order to add hooks to a specific client, call: `client.Node.Use(...)`.
func (c *Client) Use(hooks ...Hook) {
	c.Document.Use(hooks...)
	c.DocumentBlock.Use(hooks...)
	c.Library.Use(hooks...)
}

// Intercept adds the query interceptors to all the entity clients.
// In order to add interceptors to a specific client, call: `client.Node.Intercept(...)`.
func (c *Client) Intercept(interceptors ...Interceptor) {
	c.Document.Intercept(interceptors...)
	c.DocumentBlock.Intercept(interceptors...)
	c.Library.Intercept(interceptors...)
}

// Mutate implements the ent.Mutator interface.
func (c *Client) Mutate(ctx context.Context, m Mutation) (Value, error) {
	switch m := m.(type) {
	case *DocumentMutation:
		return c.Document.mutate(ctx, m)
	case *DocumentBlockMutation:
		return c.DocumentBlock.mutate(ctx, m)
	case *LibraryMutation:
		return c.Library.mutate(ctx, m)
	default:
		return nil, fmt.Errorf("ent: unknown mutation type %T", m)
	}
}

// DocumentClient is a client for the Document schema.
type DocumentClient struct {
	config
}

// NewDocumentClient returns a client for the Document from the given config.
func NewDocumentClient(c config) *DocumentClient {
	return &DocumentClient{config: c}
}

// Use adds a list of mutation hooks to the hooks stack.
// A call to `Use(f, g, h)` equals to `document.Hooks(f(g(h())))`.
func (c *DocumentClient) Use(hooks ...Hook) {
	c.hooks.Document = append(c.hooks.Document, hooks...)
}

// Intercept adds a list of query interceptors to the interceptors stack.
// A call to `Intercept(f, g, h)` equals to `document.Intercept(f(g(h())))`.
func (c *DocumentClient) Intercept(interceptors ...Interceptor) {
	c.inters.Document = append(c.inters.Document, interceptors...)
}

// Create returns a builder for creating a Document entity.
func (c *DocumentClient) Create() *DocumentCreate {
	mutation := newDocumentMutation(c.config, OpCreate)
	return &DocumentCreate{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// CreateBulk returns a builder for creating a bulk of Document entities.
func (c *DocumentClient) CreateBulk(builders ...*DocumentCreate) *DocumentCreateBulk {
	return &DocumentCreateBulk{config: c.config, builders: builders}
}

// MapCreateBulk creates a bulk creation builder from the given slice. For each item in the slice, the function creates
// a builder and applies setFunc on it.
func (c *DocumentClient) MapCreateBulk(slice any, setFunc func(*DocumentCreate, int)) *DocumentCreateBulk {
	rv := reflect.ValueOf(slice)
	if rv.Kind() != reflect.Slice {
		return &DocumentCreateBulk{err: fmt.Errorf("calling to DocumentClient.MapCreateBulk with wrong type %T, need slice", slice)}
	}
	builders := make([]*DocumentCreate, rv.Len())
	for i := 0; i < rv.Len(); i++ {
		builders[i] = c.Create()
		setFunc(builders[i], i)
	}
	return &DocumentCreateBulk{config: c.config, builders: builders}
}

// Update returns an update builder for Document.
func (c *DocumentClient) Update() *DocumentUpdate {
	mutation := newDocumentMutation(c.config, OpUpdate)
	return &DocumentUpdate{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// UpdateOne returns an update builder for the given entity.
func (c *DocumentClient) UpdateOne(d *Document) *DocumentUpdateOne {
	mutation := newDocumentMutation(c.config, OpUpdateOne, withDocument(d))
	return &DocumentUpdateOne{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// UpdateOneID returns an update builder for the given id.
func (c *DocumentClient) UpdateOneID(id int) *DocumentUpdateOne {
	mutation := newDocumentMutation(c.config, OpUpdateOne, withDocumentID(id))
	return &DocumentUpdateOne{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// Delete returns a delete builder for Document.
func (c *DocumentClient) Delete() *DocumentDelete {
	mutation := newDocumentMutation(c.config, OpDelete)
	return &DocumentDelete{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// DeleteOne returns a builder for deleting the given entity.
func (c *DocumentClient) DeleteOne(d *Document) *DocumentDeleteOne {
	return c.DeleteOneID(d.ID)
}

// DeleteOneID returns a builder for deleting the given entity by its id.
func (c *DocumentClient) DeleteOneID(id int) *DocumentDeleteOne {
	builder := c.Delete().Where(document.ID(id))
	builder.mutation.id = &id
	builder.mutation.op = OpDeleteOne
	return &DocumentDeleteOne{builder}
}

// Query returns a query builder for Document.
func (c *DocumentClient) Query() *DocumentQuery {
	return &DocumentQuery{
		config: c.config,
		ctx:    &QueryContext{Type: TypeDocument},
		inters: c.Interceptors(),
	}
}

// Get returns a Document entity by its id.
func (c *DocumentClient) Get(ctx context.Context, id int) (*Document, error) {
	return c.Query().Where(document.ID(id)).Only(ctx)
}

// GetX is like Get, but panics if an error occurs.
func (c *DocumentClient) GetX(ctx context.Context, id int) *Document {
	obj, err := c.Get(ctx, id)
	if err != nil {
		panic(err)
	}
	return obj
}

// Hooks returns the client hooks.
func (c *DocumentClient) Hooks() []Hook {
	return c.hooks.Document
}

// Interceptors returns the client interceptors.
func (c *DocumentClient) Interceptors() []Interceptor {
	return c.inters.Document
}

func (c *DocumentClient) mutate(ctx context.Context, m *DocumentMutation) (Value, error) {
	switch m.Op() {
	case OpCreate:
		return (&DocumentCreate{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpUpdate:
		return (&DocumentUpdate{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpUpdateOne:
		return (&DocumentUpdateOne{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpDelete, OpDeleteOne:
		return (&DocumentDelete{config: c.config, hooks: c.Hooks(), mutation: m}).Exec(ctx)
	default:
		return nil, fmt.Errorf("ent: unknown Document mutation op: %q", m.Op())
	}
}

// DocumentBlockClient is a client for the DocumentBlock schema.
type DocumentBlockClient struct {
	config
}

// NewDocumentBlockClient returns a client for the DocumentBlock from the given config.
func NewDocumentBlockClient(c config) *DocumentBlockClient {
	return &DocumentBlockClient{config: c}
}

// Use adds a list of mutation hooks to the hooks stack.
// A call to `Use(f, g, h)` equals to `documentblock.Hooks(f(g(h())))`.
func (c *DocumentBlockClient) Use(hooks ...Hook) {
	c.hooks.DocumentBlock = append(c.hooks.DocumentBlock, hooks...)
}

// Intercept adds a list of query interceptors to the interceptors stack.
// A call to `Intercept(f, g, h)` equals to `documentblock.Intercept(f(g(h())))`.
func (c *DocumentBlockClient) Intercept(interceptors ...Interceptor) {
	c.inters.DocumentBlock = append(c.inters.DocumentBlock, interceptors...)
}

// Create returns a builder for creating a DocumentBlock entity.
func (c *DocumentBlockClient) Create() *DocumentBlockCreate {
	mutation := newDocumentBlockMutation(c.config, OpCreate)
	return &DocumentBlockCreate{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// CreateBulk returns a builder for creating a bulk of DocumentBlock entities.
func (c *DocumentBlockClient) CreateBulk(builders ...*DocumentBlockCreate) *DocumentBlockCreateBulk {
	return &DocumentBlockCreateBulk{config: c.config, builders: builders}
}

// MapCreateBulk creates a bulk creation builder from the given slice. For each item in the slice, the function creates
// a builder and applies setFunc on it.
func (c *DocumentBlockClient) MapCreateBulk(slice any, setFunc func(*DocumentBlockCreate, int)) *DocumentBlockCreateBulk {
	rv := reflect.ValueOf(slice)
	if rv.Kind() != reflect.Slice {
		return &DocumentBlockCreateBulk{err: fmt.Errorf("calling to DocumentBlockClient.MapCreateBulk with wrong type %T, need slice", slice)}
	}
	builders := make([]*DocumentBlockCreate, rv.Len())
	for i := 0; i < rv.Len(); i++ {
		builders[i] = c.Create()
		setFunc(builders[i], i)
	}
	return &DocumentBlockCreateBulk{config: c.config, builders: builders}
}

// Update returns an update builder for DocumentBlock.
func (c *DocumentBlockClient) Update() *DocumentBlockUpdate {
	mutation := newDocumentBlockMutation(c.config, OpUpdate)
	return &DocumentBlockUpdate{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// UpdateOne returns an update builder for the given entity.
func (c *DocumentBlockClient) UpdateOne(db *DocumentBlock) *DocumentBlockUpdateOne {
	mutation := newDocumentBlockMutation(c.config, OpUpdateOne, withDocumentBlock(db))
	return &DocumentBlockUpdateOne{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// UpdateOneID returns an update builder for the given id.
func (c *DocumentBlockClient) UpdateOneID(id int) *DocumentBlockUpdateOne {
	mutation := newDocumentBlockMutation(c.config, OpUpdateOne, withDocumentBlockID(id))
	return &DocumentBlockUpdateOne{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// Delete returns a delete builder for DocumentBlock.
func (c *DocumentBlockClient) Delete() *DocumentBlockDelete {
	mutation := newDocumentBlockMutation(c.config, OpDelete)
	return &DocumentBlockDelete{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// DeleteOne returns a builder for deleting the given entity.
func (c *DocumentBlockClient) DeleteOne(db *DocumentBlock) *DocumentBlockDeleteOne {
	return c.DeleteOneID(db.ID)
}

// DeleteOneID returns a builder for deleting the given entity by its id.
func (c *DocumentBlockClient) DeleteOneID(id int) *DocumentBlockDeleteOne {
	builder := c.Delete().Where(documentblock.ID(id))
	builder.mutation.id = &id
	builder.mutation.op = OpDeleteOne
	return &DocumentBlockDeleteOne{builder}
}

// Query returns a query builder for DocumentBlock.
func (c *DocumentBlockClient) Query() *DocumentBlockQuery {
	return &DocumentBlockQuery{
		config: c.config,
		ctx:    &QueryContext{Type: TypeDocumentBlock},
		inters: c.Interceptors(),
	}
}

// Get returns a DocumentBlock entity by its id.
func (c *DocumentBlockClient) Get(ctx context.Context, id int) (*DocumentBlock, error) {
	return c.Query().Where(documentblock.ID(id)).Only(ctx)
}

// GetX is like Get, but panics if an error occurs.
func (c *DocumentBlockClient) GetX(ctx context.Context, id int) *DocumentBlock {
	obj, err := c.Get(ctx, id)
	if err != nil {
		panic(err)
	}
	return obj
}

// QueryDocument queries the document edge of a DocumentBlock.
func (c *DocumentBlockClient) QueryDocument(db *DocumentBlock) *DocumentQuery {
	query := (&DocumentClient{config: c.config}).Query()
	query.path = func(context.Context) (fromV *sql.Selector, _ error) {
		id := db.ID
		step := sqlgraph.NewStep(
			sqlgraph.From(documentblock.Table, documentblock.FieldID, id),
			sqlgraph.To(document.Table, document.FieldID),
			sqlgraph.Edge(sqlgraph.O2M, false, documentblock.DocumentTable, documentblock.DocumentColumn),
		)
		fromV = sqlgraph.Neighbors(db.driver.Dialect(), step)
		return fromV, nil
	}
	return query
}

// Hooks returns the client hooks.
func (c *DocumentBlockClient) Hooks() []Hook {
	return c.hooks.DocumentBlock
}

// Interceptors returns the client interceptors.
func (c *DocumentBlockClient) Interceptors() []Interceptor {
	return c.inters.DocumentBlock
}

func (c *DocumentBlockClient) mutate(ctx context.Context, m *DocumentBlockMutation) (Value, error) {
	switch m.Op() {
	case OpCreate:
		return (&DocumentBlockCreate{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpUpdate:
		return (&DocumentBlockUpdate{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpUpdateOne:
		return (&DocumentBlockUpdateOne{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpDelete, OpDeleteOne:
		return (&DocumentBlockDelete{config: c.config, hooks: c.Hooks(), mutation: m}).Exec(ctx)
	default:
		return nil, fmt.Errorf("ent: unknown DocumentBlock mutation op: %q", m.Op())
	}
}

// LibraryClient is a client for the Library schema.
type LibraryClient struct {
	config
}

// NewLibraryClient returns a client for the Library from the given config.
func NewLibraryClient(c config) *LibraryClient {
	return &LibraryClient{config: c}
}

// Use adds a list of mutation hooks to the hooks stack.
// A call to `Use(f, g, h)` equals to `library.Hooks(f(g(h())))`.
func (c *LibraryClient) Use(hooks ...Hook) {
	c.hooks.Library = append(c.hooks.Library, hooks...)
}

// Intercept adds a list of query interceptors to the interceptors stack.
// A call to `Intercept(f, g, h)` equals to `library.Intercept(f(g(h())))`.
func (c *LibraryClient) Intercept(interceptors ...Interceptor) {
	c.inters.Library = append(c.inters.Library, interceptors...)
}

// Create returns a builder for creating a Library entity.
func (c *LibraryClient) Create() *LibraryCreate {
	mutation := newLibraryMutation(c.config, OpCreate)
	return &LibraryCreate{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// CreateBulk returns a builder for creating a bulk of Library entities.
func (c *LibraryClient) CreateBulk(builders ...*LibraryCreate) *LibraryCreateBulk {
	return &LibraryCreateBulk{config: c.config, builders: builders}
}

// MapCreateBulk creates a bulk creation builder from the given slice. For each item in the slice, the function creates
// a builder and applies setFunc on it.
func (c *LibraryClient) MapCreateBulk(slice any, setFunc func(*LibraryCreate, int)) *LibraryCreateBulk {
	rv := reflect.ValueOf(slice)
	if rv.Kind() != reflect.Slice {
		return &LibraryCreateBulk{err: fmt.Errorf("calling to LibraryClient.MapCreateBulk with wrong type %T, need slice", slice)}
	}
	builders := make([]*LibraryCreate, rv.Len())
	for i := 0; i < rv.Len(); i++ {
		builders[i] = c.Create()
		setFunc(builders[i], i)
	}
	return &LibraryCreateBulk{config: c.config, builders: builders}
}

// Update returns an update builder for Library.
func (c *LibraryClient) Update() *LibraryUpdate {
	mutation := newLibraryMutation(c.config, OpUpdate)
	return &LibraryUpdate{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// UpdateOne returns an update builder for the given entity.
func (c *LibraryClient) UpdateOne(l *Library) *LibraryUpdateOne {
	mutation := newLibraryMutation(c.config, OpUpdateOne, withLibrary(l))
	return &LibraryUpdateOne{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// UpdateOneID returns an update builder for the given id.
func (c *LibraryClient) UpdateOneID(id int) *LibraryUpdateOne {
	mutation := newLibraryMutation(c.config, OpUpdateOne, withLibraryID(id))
	return &LibraryUpdateOne{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// Delete returns a delete builder for Library.
func (c *LibraryClient) Delete() *LibraryDelete {
	mutation := newLibraryMutation(c.config, OpDelete)
	return &LibraryDelete{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// DeleteOne returns a builder for deleting the given entity.
func (c *LibraryClient) DeleteOne(l *Library) *LibraryDeleteOne {
	return c.DeleteOneID(l.ID)
}

// DeleteOneID returns a builder for deleting the given entity by its id.
func (c *LibraryClient) DeleteOneID(id int) *LibraryDeleteOne {
	builder := c.Delete().Where(library.ID(id))
	builder.mutation.id = &id
	builder.mutation.op = OpDeleteOne
	return &LibraryDeleteOne{builder}
}

// Query returns a query builder for Library.
func (c *LibraryClient) Query() *LibraryQuery {
	return &LibraryQuery{
		config: c.config,
		ctx:    &QueryContext{Type: TypeLibrary},
		inters: c.Interceptors(),
	}
}

// Get returns a Library entity by its id.
func (c *LibraryClient) Get(ctx context.Context, id int) (*Library, error) {
	return c.Query().Where(library.ID(id)).Only(ctx)
}

// GetX is like Get, but panics if an error occurs.
func (c *LibraryClient) GetX(ctx context.Context, id int) *Library {
	obj, err := c.Get(ctx, id)
	if err != nil {
		panic(err)
	}
	return obj
}

// Hooks returns the client hooks.
func (c *LibraryClient) Hooks() []Hook {
	return c.hooks.Library
}

// Interceptors returns the client interceptors.
func (c *LibraryClient) Interceptors() []Interceptor {
	return c.inters.Library
}

func (c *LibraryClient) mutate(ctx context.Context, m *LibraryMutation) (Value, error) {
	switch m.Op() {
	case OpCreate:
		return (&LibraryCreate{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpUpdate:
		return (&LibraryUpdate{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpUpdateOne:
		return (&LibraryUpdateOne{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpDelete, OpDeleteOne:
		return (&LibraryDelete{config: c.config, hooks: c.Hooks(), mutation: m}).Exec(ctx)
	default:
		return nil, fmt.Errorf("ent: unknown Library mutation op: %q", m.Op())
	}
}

// hooks and interceptors per client, for fast access.
type (
	hooks struct {
		Document, DocumentBlock, Library []ent.Hook
	}
	inters struct {
		Document, DocumentBlock, Library []ent.Interceptor
	}
)
